/*
 * eqlinfo: X application for displaying eql load balancing
 *
 */

/* 
 * Based upon sysinfo, written by Gabor Herr <herr@iti.informatik.th-darmstadt.de>.
 * eqlinfo adaptation was written by cWare, Inc.
 *
 * Copyright (c) 1992, 1993 by Gabor Herr, all rights reserved.
 * Copyright (c) 1999 by cWare, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that may name is not used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission. I make no representations
 * about the suitability of this software for any purpose. It is
 * provided "as is" without express or implied warranty.
 */

#include "eqlinfo.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

static FILE *devinfo_fh;

static char buffer[1024];

char *safe_strncpy(char *dst, const char *src, size_t size)
{
	dst[size - 1] = '\0';
	return strncpy(dst, src, size - 1);
}
static char *get_name(char *name, char *p)
{
	while (isspace(*p))
		p++;
	while (*p) {
		if (isspace(*p))
			break;
		if (*p == ':') {	/* could be an alias */
			char *dot = p, *dotname = name;
			*name++ = *p++;
			while (isdigit(*p))
				*name++ = *p++;
			if (*p != ':') {	/* it wasn't, backup */
				p = dot;
				name = dotname;
			}
			if (*p == '\0')
				return NULL;
			p++;
			break;
		}
		*name++ = *p++;
	}
	*name++ = '\0';
	return p;
}
static int get_dev_stats(char *bp, struct Stats *stats, int version)
{
	long int dummy;

	switch (version) {
		case 3:
			sscanf(bp,
					"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
					&stats->rx_bytes,
					&stats->rx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy,

					&stats->tx_bytes,
					&stats->tx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy);
			break;
		case 2:
			sscanf(bp, "%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
					&stats->rx_bytes,
					&stats->rx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,

					&stats->tx_bytes,
					&stats->tx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy);
			break;
		case 1:
			sscanf(bp, "%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
					&stats->rx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,

					&stats->tx_packets,
					&dummy,
					&dummy,
					&dummy,
					&dummy,
					&dummy);
			stats->rx_bytes = 0;
			stats->tx_bytes = 0;
			break;
	}
	return 0;
}

void get_devinfo(struct Stats *result)
{
	int i;
	char *bufptr;
	int res;
	int sw_size, sw_used;
	char buf[512];
	char thisName[512];
	int version;

	if (fseek(devinfo_fh, 0L, 0) < 0) {
		perror("get_devinfo");
		exit(1);
	}
	fgets(buf, sizeof buf, devinfo_fh);	/* eat line */
	fgets(buf, sizeof buf, devinfo_fh);

	version = 1;
	if (strstr(buf, "compressed"))
		version = 3;
	else if (strstr(buf, "bytes"))
		version = 2;

	while (fgets(buf, sizeof buf, devinfo_fh)) {
		char *s;
		char ppp_name[256];
		int i;

		s = get_name(thisName, buf);
		if (!strcmp(thisName, "eql")) {
			get_dev_stats(s, &result[EQL_STATS], version);
		} else {
			for (i = 0; i < MAX_STATS - 1; i++) {
				sprintf(ppp_name,"ppp%i",i);
				if (!strcmp(thisName, ppp_name)) {
					get_dev_stats(s, &result[i+1], version);
				}
			}
		}
	}
}

int eqlinfo_init(void)
{
	if ((devinfo_fh = fopen("/proc/net/dev", "r")) < 0) {
		perror("/proc/net/dev");
		return 1;
	}
	return 0;
}
